/*
* @(#)ClassServer.java - Server per un NetworkClassLoader.
*
*/

package loader;

import java.io.*;
import java.net.*;
import java.util.Hashtable;
import loader.WorkerClassServer;
/**
* <br>
* Questa classe assieme  alla classe WorkerClassLoader rappresenta la 
* parte server del NetworkClassLoader. <br>
* <br>
* La classe, in esecuzione in un thread, e una  "classica" classe che usa la classe java.net.ServerSocket. <br>
* Essa sta in attesa di una connessione su una determinata porta (default 5050).<br>
* Per ogni nuova connessione viene generato un nuovo thread di tipo WorkerClassLoader
* a cui vengono passati i parametri:<br>
* clientSocket - Il socket restituito dalla connessione appena accettata.<br>
* classesChace - Una cache, globale al server, delle classi gia' caricate.<br>
* <br>
* @see			java.lang.Thread
* @version 	1.0 Gennaio 1998
* @author		Donato Cappetta - cappetta@diiie.unisa.it
*/

public class ClassServer extends Thread {

	/**
	* Variabile per  acquisire il valore della porta di ascolto
	* dalla linea di comandi.
	* Valore di default 5050.
	*/
	private int port = 5050;

	/**
	* serverSocket per l'ascolto della connessione.
	*/
	private ServerSocket serverSocket = null;


	/**
	* Nome e indirizzo della macchina su cui  classServer va in esecuzione.
	*/
	private InetAddress inetAddress = null;
	/**
	* classChace dovrebbe servire per creare una cache di classi sul server.
	*/
    private Hashtable classesCache = null;


	/**
	* Costruttore.
	*
	* @param    port    Porta di ascolto per il serverSocket, se port = 0
	* si considera la porta di default (5050).
	* @exception	 UnknownHostException Non e' associato nessun nome alla macchina.
	* @exception   IOException		Errore di I/O.
	*/
	public ClassServer(int port) throws UnknownHostException,IOException {
		super("ClassServer");
		if ((port > 0x0) && (port <= 0xFFFF)) this.port = port;
		try {
			inetAddress = InetAddress.getLocalHost();
			System.out.println("ClassServer start on  " + inetAddress + ":"+ this.port + " ...");
			serverSocket = new ServerSocket(this.port);
		} catch (UnknownHostException uhe) {
			throw uhe;
		} catch (IOException ioe) {
			throw ioe;
		}	//end try catch()
		classesCache = new Hashtable();
	}	//end costruttore ClassServer().


/**
* Un ciclo infinito che ascolta e accetta le connessioni.<br>
* Per ogni connesione accettata viene creato un nuovo thread che
* gestisce la comunicazione con il client.<br>
*/

public void run() {
	while (true) {
		System.out.println("ClassServer in attesa sulla porta " + port + " ... ");
		Socket clientSocket = null;
		try {
			clientSocket = serverSocket.accept();
			System.out.println("\nClient connesso: " + clientSocket.getInetAddress());
			WorkerClassServer wcs  = new WorkerClassServer(clientSocket, classesCache);
			wcs.start();
		} catch (IOException ioe) {
			System.out.println("Connessione fallita " + ioe.getMessage());
			continue;
		} catch(Throwable t){
			continue;
		}	//end try-catch()
	}	//end while()
}	//end run()

 /**
 * Svuota la cache.
 */
  protected void finalize(){
      	classesCache.clear();
   		classesCache = null;
   		try {
  			super.finalize();
  		} catch (Throwable t) {}
  }	//end finalize()

} // end class ClassServer